@@ -8,7 +8,7 @@
     Copyright 2020-2021 Kristian Amlie, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -24,7 +24,6 @@
     This file is derivative of ZynAddSubFX original code
 */
 
-#include "DSP/FFTwrapper.h"
 #include "DSP/FormantFilter.h"
 #include "Misc/SynthEngine.h"
 #include "Misc/SynthHelper.h"
@@ -37,17 +36,17 @@ using func::powFrac;
 using func::power;
 
 
-FormantFilter::FormantFilter(FilterParams *pars_, SynthEngine *_synth):
+FormantFilter::FormantFilter(SynthEngine* _synth, FilterParams* pars_):
     pars(pars_),
-    parsUpdate(pars_),
-    synth(_synth)
+    parsUpdate(*pars_),
+    synth(_synth),
+    inbuffer(synth->buffersize),
+    tmpbuff (synth->buffersize)
 {
     numformants = pars->Pnumformants;
     for (int i = 0; i < numformants; ++i)
-        formant[i] = new AnalogFilter(4/*BPF*/, 1000.0f, 10.0f, pars->Pstages, synth);
+        formant[i] = new AnalogFilter(*synth, TOPLEVEL::filter::Band2, 1000.0f, 10.0f, pars->Pstages);
     cleanup();
-    inbuffer = (float*)fftwf_malloc(synth->bufferbytes);
-    tmpbuf = (float*)fftwf_malloc(synth->bufferbytes);
 
     for (int i = 0; i < FF_MAX_FORMANTS; ++i)
         oldformantamp[i] = 1.0f;
@@ -88,7 +87,10 @@ FormantFilter::FormantFilter(const FormantFilter &orig) :
     oldQfactor(orig.oldQfactor),
     vowelclearness(orig.vowelclearness),
     sequencestretch(orig.sequencestretch),
-    synth(orig.synth)
+    synth(orig.synth),
+    // These don't hold persistent state and don't need a memcpy
+    inbuffer(synth->buffersize),
+    tmpbuff (synth->buffersize)
 {
     outgain = orig.outgain;
 
@@ -99,10 +101,6 @@ FormantFilter::FormantFilter(const FormantFilter &orig) :
 
     for (int i = 0; i < numformants; ++i)
         formant[i] = new AnalogFilter(*orig.formant[i]);
-
-    // These don't hold persistent state and don't need a memcpy
-    inbuffer = (float*)fftwf_malloc(synth->bufferbytes);
-    tmpbuf = (float*)fftwf_malloc(synth->bufferbytes);
 }
 
 
@@ -110,8 +108,6 @@ FormantFilter::~FormantFilter()
 {
     for (int i = 0; i < numformants; ++i)
         delete(formant[i]);
-    fftwf_free(inbuffer);
-    fftwf_free(tmpbuf);
 }
 
 
@@ -141,7 +137,8 @@ void FormantFilter::setpos(float input)
         //	oldinput=input; daca setez asta, o sa faca probleme la schimbari foarte lente
         firsttime = 0;
         return;
-    } else
+    }
+    else
         oldinput = input;
 
     float pos = input * sequencestretch;
@@ -175,7 +172,9 @@ void FormantFilter::setpos(float input)
             oldformantamp[i] = currentformants[i].amp;
         }
         firsttime = 0;
-    } else {
+    }
+    else
+    {
         for (int i = 0; i < numformants; ++i)
         {
             currentformants[i].freq =
@@ -204,12 +203,14 @@ void FormantFilter::setpos(float input)
 void FormantFilter::updateCurrentParameters()
 {
     for (int j = 0; j < FF_MAX_VOWELS; ++j)
+    {
         for (int i = 0; i < numformants; ++i)
         {
             formantpar[j][i].freq = pars->getformantfreq(pars->Pvowels[j].formants[i].freq);
             formantpar[j][i].amp = pars->getformantamp(pars->Pvowels[j].formants[i].amp);
             formantpar[j][i].q = pars->getformantq(pars->Pvowels[j].formants[i].q);
         }
+    }
 
     formantslowness = powf(1.0f - (pars->Pformantslowness / 128.0f), 3.0f);
 
@@ -248,24 +249,28 @@ void FormantFilter::setfreq_and_q(float frequency, float q_)
 
 void FormantFilter::filterout(float *smp)
 {
-    memcpy(inbuffer, smp, synth->sent_bufferbytes);
+    memcpy(inbuffer.get(), smp, synth->sent_bufferbytes);
     memset(smp, 0, synth->sent_bufferbytes);
 
     for (int j = 0; j < numformants; ++j)
     {
         for (int k = 0; k < synth->sent_buffersize; ++k)
-            tmpbuf[k] = inbuffer[k] * outgain;
-        formant[j]->filterout(tmpbuf);
+            tmpbuff[k] = inbuffer[k] * outgain;
+        formant[j]->filterout(tmpbuff.get());
 
         if (aboveAmplitudeThreshold(oldformantamp[j], currentformants[j].amp))
+        {
             for (int i = 0; i < synth->sent_buffersize; ++i)
-                smp[i] += tmpbuf[i]
+                smp[i] += tmpbuff[i]
                           * interpolateAmplitude(oldformantamp[j],
                                                   currentformants[j].amp, i,
                                                   synth->sent_buffersize);
+        }
         else
+        {
             for (int i = 0; i < synth->sent_buffersize; ++i)
-                smp[i] += tmpbuf[i] * currentformants[j].amp;
+                smp[i] += tmpbuff[i] * currentformants[j].amp;
+        }
         oldformantamp[j] = currentformants[j].amp;
     }
 }
