@@ -7,7 +7,7 @@
     Copyright 2020, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -29,26 +29,42 @@
 
 using func::setAllPan;
 
+namespace {
+    const float PAN_NORMAL_CENTRE = cosf(0.5f * HALFPI);
+}
+
 Effect::Effect(bool insertion_, float *efxoutl_, float *efxoutr_,
-               FilterParams *filterpars_, unsigned char Ppreset_,
-               SynthEngine *synth_) :
-    Ppreset(Ppreset_),
-    efxoutl(efxoutl_),
-    efxoutr(efxoutr_),
-    outvolume(0.5f, synth_->samplerate),
-    volume(0.5f, synth_->samplerate),
-    filterpars(filterpars_),
-    insertion(insertion_),
-    pangainL(0.5f, synth_->samplerate),
-    pangainR(0.5f, synth_->samplerate),
-    lrcross(0.5f, synth_->samplerate),
-    synth(synth_)
+               FilterParams *filterpars_, uchar Ppreset_,
+               SynthEngine& synth_)
+    : Ppreset{Ppreset_}
+    , efxoutl{efxoutl_}
+    , efxoutr{efxoutr_}
+    , outvolume{0.5f, synth_.samplerate}
+    , volume{0.5f, synth_.samplerate}
+    , filterpars{filterpars_}
+    , insertion{insertion_}
+    , pangainL{PAN_NORMAL_CENTRE, synth_.samplerate}
+    , pangainR{PAN_NORMAL_CENTRE, synth_.samplerate}
+    , lrcross{40.0f/127, synth_.samplerate}
+    , synth{synth_}
 {
     setpanning(64);
     setlrcross(40);
 }
 
 
+// base implementation: force to clean reproducible state.
+// Note: be sure to invoke that from overridden implementations.
+void Effect::cleanup()
+{
+    outvolume.pushToTarget();
+    volume   .pushToTarget();
+    pangainL.pushToTarget();
+    pangainR.pushToTarget();
+    lrcross.pushToTarget();
+}
+
+
 void Effect::setpanning(char Ppanning_)
 {
     Ppanning = Ppanning_;
@@ -71,3 +87,16 @@ void Effect::setlrcross(char Plrcross_)
     Plrcross = Plrcross_;
     lrcross.setTargetValue(Plrcross / 127.0f);
 }
+
+
+/**
+ * Base implementation: fetch through `virtual getpar(n)`
+ * Only the EQ uses the high number of (band) parameters.
+ */
+void Effect::getAllPar(EffectParArray& target) const
+{
+    const uint LIM = EFFECT::control::bpmStart;
+    for (uint i=0; i<=LIM; ++i)
+        target[i] = this->getpar(i);
+}
+
