@@ -7,7 +7,7 @@
     Copyright 2018-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -26,9 +26,6 @@
 
 #include <cmath>
 
-using namespace std;
-
-#include "DSP/FFTwrapper.h"
 #include "DSP/Unison.h"
 #include "DSP/AnalogFilter.h"
 #include "Misc/SynthEngine.h"
@@ -38,40 +35,9 @@ using namespace std;
 using func::power;
 using func::powFrac;
 
-static const int PRESET_SIZE = 13;
-static const int NUM_PRESETS = 13;
-static unsigned char presets[NUM_PRESETS][PRESET_SIZE] = {
-        // Cathedral1
-        {80,  64,  63,  24,  0,  0,  0, 85,  5,  83,   1,  64,  20 },
-        // Cathedral2
-        {80,  64,  69,  35,  0,  0,  0, 127, 0,  71,   0,  64,  20 },
-        // Cathedral3
-        {80,  64,  69,  24,  0,  0,  0, 127, 75, 78,   1,  85,  20 },
-        // Hall1
-        {90,  64,  51,  10,  0,  0,  0, 127, 21, 78,   1,  64,  20 },
-        // Hall2
-        {90,  64,  53,  20,  0,  0,  0, 127, 75, 71,   1,  64,  20 },
-        // Room1
-        {100, 64,  33,  0,   0,  0,  0, 127, 0,  106,  0,  30,  20 },
-        // Room2
-        {100, 64,  21,  26,  0,  0,  0, 62,  0,  77,   1,  45,  20 },
-        // Basement
-        {110, 64,  14,  0,   0,  0,  0, 127, 5,  71,   0,  25,  20 },
-        // Tunnel
-        {85,  80,  84,  20,  42, 0,  0, 51,  0,  78,   1,  105, 20 },
-        // Echoed1
-        {95,  64,  26,  60,  71, 0,  0, 114, 0,  64,   1,  64,  20 },
-        // Echoed2
-        {90,  64,  40,  88,  71, 0,  0, 114, 0,  88,   1,  64,  20 },
-        // VeryLong1
-        {90,  64,  93,  15,  0,  0,  0, 114, 0,  77,   0,  95,  20 },
-        // VeryLong2
-        {90,  64,  111, 30,  0,  0,  0, 114, 90, 74,   1,  80,  20 }
-};
-
 // todo: EarlyReflections, Prdelay, Perbalance
 
-Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
+Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine& _synth) :
     Effect(insertion_, efxoutl_, efxoutr_, NULL, 0, _synth),
     // defaults
 //    Pvolume(48),
@@ -92,15 +58,15 @@ Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_
     idelay(NULL),
     lpf(NULL),
     hpf(NULL), // no filter
-    lpffr(0, synth->samplerate),
-    hpffr(0, synth->samplerate)
+    lpffr(0, synth.samplerate),
+    hpffr(0, synth.samplerate),
+    inputbuf(_synth.buffersize)
 {
     setvolume(48);
-    inputbuf = (float*)fftwf_malloc(synth->bufferbytes);
     for (int i = 0; i < REV_COMBS * 2; ++i)
     {
 
-        comblen[i] = 800 + synth->randomINT() / (INT32_MAX/1400);
+        comblen[i] = 800 + synth.randomINT() / (INT32_MAX/1400);
         combk[i] = 0;
         lpcomb[i] = 0;
         combfb[i] = -0.97f;
@@ -109,7 +75,7 @@ Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_
 
     for (int i = 0; i < REV_APS * 2; ++i)
     {
-        aplen[i] = 500 + synth->randomINT() / (INT32_MAX/500);
+        aplen[i] = 500 + synth.randomINT() / (INT32_MAX/500);
         apk[i] = 0;
         ap[i] = NULL;
     }
@@ -132,100 +98,99 @@ Reverb::~Reverb()
         delete [] ap[i];
     for (i = 0; i < REV_COMBS * 2; ++i)
         delete [] comb[i];
-    fftwf_free(inputbuf);
 
     if (bandwidth)
         delete bandwidth;
 }
 
 
-// Cleanup the effect
-void Reverb::cleanup(void)
+void Reverb::clearBuffers()
 {
-    int i, j;
-    memset(lpcomb, 0, sizeof(float) * REV_COMBS * 2);
-    for (i = 0; i < REV_COMBS * 2; ++i)
+    for (size_t j = 0; j < REV_COMBS * 2; ++j)
     {
-        for (j = 0; j < comblen[i]; ++j)
-            comb[i][j] = 0.0f; // not sure how to memset this!
+        combk[j] = 0;
+        lpcomb[j] = 0.0;
+        for (size_t smp = 0; smp < comblen[j]; ++smp)
+            comb[j][smp] = 0.0f;
+    }
+    for (size_t j = 0; j < REV_APS * 2; ++j)
+    {
+        apk[j] = 0;
+        for (size_t smp = 0; smp < aplen[j]; ++smp)
+            ap[j][smp] = 0.0f;
     }
-    for (i = 0; i < REV_APS * 2; ++i)
-        for (j = 0; j < aplen[i]; ++j)
-            ap[i][j] = 0.0f;
 
     if (idelay)
         memset(idelay, 0, sizeof(float) * idelaylen);
-    if (hpf)
-        hpf->cleanup();
-    if (lpf)
-        lpf->cleanup();
 }
 
 
 // Process one channel; 0 = left, 1 = right
-void Reverb::processmono(int ch, float *output)
+void Reverb::calculateReverb(size_t ch, Samples& inputFeed, float *output)
 {
-    int i, j;
-    float fbout, tmp;
-    // \todo: implement the high part from lohidamp
+    ////TODO: implement the high part from lohidamp    (comment probably from original author, before 2010)
 
-    for (j = REV_COMBS * ch; j < REV_COMBS * (ch + 1); ++j)
+    for (size_t j = REV_COMBS * ch; j < REV_COMBS * (ch + 1); ++j)
     {
-        int ck = combk[j];
-        int comblength = comblen[j];
-        float lpcombj = lpcomb[j];
-
-        for (i = 0; i < synth->sent_buffersize; ++i)
+        size_t offset = combk[j];
+        size_t combLen = comblen[j];
+        float lowpassj = lpcomb[j];
+        for (size_t smp = 0; smp < size_t(synth.sent_buffersize); ++smp)
         {
-            fbout = comb[j][ck] * combfb[j];
-            fbout = fbout * (1.0f - lohifb) + lpcombj * lohifb;
-            lpcombj = fbout;
+            float feedback = comb[j][offset] * combfb[j];
+            feedback = feedback * (1.0f - lohifb) + lowpassj * lohifb;
+            lowpassj = feedback;
 
-            comb[j][ck] = inputbuf[i] + fbout;
-            output[i] += fbout;
+            comb[j][offset] = inputFeed[smp] + feedback;
+            output[smp] += feedback;
 
-            if ((++ck) >= comblength)
-                ck = 0;
+            if ((++offset) >= combLen)
+                offset = 0;
         }
 
-        combk[j] = ck;
-        lpcomb[j] = lpcombj;
+        combk[j] = offset;
+        lpcomb[j] = lowpassj;
     }
 
-    for (j = REV_APS * ch; j < REV_APS * (1 + ch); ++j)
+    // feed result of comb filters into AllPass filters
+    for (size_t j = REV_APS * ch; j < REV_APS * (1 + ch); ++j)
     {
-        int ak = apk[j];
-        int aplength = aplen[j];
-        for (i = 0; i < synth->sent_buffersize; ++i)
+        size_t offset = apk[j];
+        size_t allpassLen = aplen[j];
+        for (size_t smp = 0; smp < size_t(synth.sent_buffersize); ++smp)
         {
-            tmp = ap[j][ak];
-            ap[j][ak] = 0.7f * tmp + output[i];
-            output[i] = tmp - 0.7f * ap[j][ak] + 1e-20f; // anti-denormal - a very, very, very small dc bias
-            if ((++ak) >= aplength)
-                ak = 0;
+            float feedback = ap[j][offset];
+            ap[j][offset] = 0.7f * feedback + output[smp];
+            output[smp] = feedback - 0.7f * ap[j][offset] + 1e-20f; // anti-denormal - a very, very, very small dc bias
+            if ((++offset) >= allpassLen)
+                offset = 0;
         }
-        apk[j] = ak;
+        apk[j] = offset;
     }
 }
 
 
-// Effect output
-void Reverb::out(float *smps_l, float *smps_r)
-{
-    outvolume.advanceValue(synth->sent_buffersize);
+namespace { //Helper: detect change above rounding errors for frequency interpolation
 
-    if (!Pvolume && insertion)
-        return;
+    const float FREQUENCY_EPSILON = 1e-3;
 
-    int i;
-    for (i = 0; i < synth->sent_buffersize; ++i)
+    bool significantChange(float newVal, float oldVal)
     {
-        inputbuf[i] = float(1e-20) + ((smps_l[i] + smps_r[i]) / 2.0f); // includes anti-denormal
-        // Initial delay r
+        return std::fabs(newVal - oldVal) > FREQUENCY_EPSILON;
+    }
+}
+
+
+void Reverb::preprocessInput(float *rawL, float *rawR, Samples& inputFeed)
+{
+    for (size_t i = 0; i < size_t(synth.sent_buffersize); ++i)
+    {
+        inputFeed[i] = float(1e-20) + ((rawL[i] + rawR[i]) / 2.0f); // includes anti-denormal
+
         if (idelay)
-        {
-            float tmp = inputbuf[i] + idelay[idelayk] * idelayfb;
-            inputbuf[i] = idelay[idelayk];
+        {// shift input by pre-delay
+            float tmp = inputFeed[i] + idelay[idelayk] * idelayfb;
+            inputFeed[i] = idelay[idelayk];
             idelay[idelayk] = tmp;
             idelayk++;
             if (idelayk >= idelaylen)
@@ -234,33 +199,46 @@ void Reverb::out(float *smps_l, float *smps_r)
     }
 
     if (bandwidth)
-        bandwidth->process(synth->sent_buffersize, inputbuf);
+        bandwidth->process(synth.sent_buffersize, inputFeed.get());
 
     if (lpf)
     {
-        float fr = lpffr.getValue();
-        lpffr.advanceValue(synth->sent_buffersize);
-        if (fr != lpffr.getValue())
+        float currFreq = lpf->getFreq();
+        lpffr.advanceValue(synth.sent_buffersize);
+        if (significantChange(currFreq, lpffr.getValue()))
         {
             lpf->interpolatenextbuffer();
             lpf->setfreq(lpffr.getValue());
         }
-        lpf->filterout(inputbuf);
+        lpf->filterout(inputFeed.get());
     }
      if (hpf)
     {
-        float fr = hpffr.getValue();
-        hpffr.advanceValue(synth->sent_buffersize);
-        if (fr != hpffr.getValue())
+        float currFreq = hpf->getFreq();
+        hpffr.advanceValue(synth.sent_buffersize);
+        if (significantChange(currFreq, hpffr.getValue()))
         {
             hpf->interpolatenextbuffer();
             hpf->setfreq(hpffr.getValue());
         }
-         hpf->filterout(inputbuf);
+         hpf->filterout(inputFeed.get());
     }
+}
 
-    processmono(0, efxoutl); // left
-    processmono(1, efxoutr); // right
+
+
+// Effect output
+void Reverb::out(float *rawL, float *rawR)
+{
+    outvolume.advanceValue(synth.sent_buffersize);
+
+    if (!Pvolume && insertion)
+        return;
+
+    preprocessInput(rawL,rawR, inputbuf);
+
+    calculateReverb(0, inputbuf, efxoutl); // inputbuf -> left
+    calculateReverb(1, inputbuf, efxoutr); // inputbuf -> right
 
     float lvol = rs / REV_COMBS * pangainL.getAndAdvanceValue();
     float rvol = rs / REV_COMBS * pangainR.getAndAdvanceValue();
@@ -269,7 +247,7 @@ void Reverb::out(float *smps_l, float *smps_r)
         lvol *= 2.0f;
         rvol *= 2.0f;
     }
-    for (i = 0; i < synth->sent_buffersize; ++i)
+    for (size_t i = 0; i < size_t(synth.sent_buffersize); ++i)
     {
         efxoutl[i] *= lvol;
         efxoutr[i] *= rvol;
@@ -277,8 +255,24 @@ void Reverb::out(float *smps_l, float *smps_r)
 }
 
 
+// Reset the effect to pristine state
+void Reverb::cleanup()
+{
+    Effect::cleanup();
+    lpffr.pushToTarget();
+    hpffr.pushToTarget();
+    setupPipelines();
+    settime(Ptime);
+    clearBuffers();
+    if (hpf)
+        hpf->cleanup();
+    if (lpf)
+        lpf->cleanup();
+}
+
+
 // Parameter control
-void Reverb::setvolume(unsigned char Pvolume_)
+void Reverb::setvolume(uchar Pvolume_)
 {
     Pvolume = Pvolume_;
     if (!insertion)
@@ -297,17 +291,17 @@ void Reverb::setvolume(unsigned char Pvolume_)
 }
 
 
-void Reverb::settime(unsigned char Ptime_)
+void Reverb::settime(uchar Ptime_)
 {
     Ptime = Ptime_;
     float t = power<60>(Ptime / 127.0f) - 0.97f;
     for (int i = 0; i < REV_COMBS * 2; ++i)
-        combfb[i] = -expf((float)comblen[i] / synth->samplerate_f * logf(0.001f) / t);
+        combfb[i] = -expf(float(comblen[i]) / synth.samplerate_f * logf(0.001f) / t);
         // the feedback is negative because it removes the DC
 }
 
 
-void Reverb::setlohidamp(unsigned char Plohidamp_)
+void Reverb::setlohidamp(uchar Plohidamp_)
 {
     Plohidamp = (Plohidamp_ < 64) ? 64 : Plohidamp_;
                        // remove this when the high part from lohidamp is added
@@ -328,7 +322,7 @@ void Reverb::setlohidamp(unsigned char Plohidamp_)
 }
 
 
-void Reverb::setidelay(unsigned char Pidelay_)
+void Reverb::setidelay(uchar Pidelay_)
 {
     Pidelay = Pidelay_;
     float delay = powf(50.0f * Pidelay / 127.0f, 2.0f) - 1.0f;
@@ -337,7 +331,7 @@ void Reverb::setidelay(unsigned char Pidelay_)
         delete [] idelay;
     idelay = NULL;
 
-    idelaylen = lrint(synth->samplerate_f * delay / 1000.0f);
+    idelaylen = lrint(synth.samplerate_f * delay / 1000.0f);
     if (idelaylen > 1)
     {
         idelayk = 0;
@@ -347,14 +341,14 @@ void Reverb::setidelay(unsigned char Pidelay_)
 }
 
 
-void Reverb::setidelayfb(unsigned char Pidelayfb_)
+void Reverb::setidelayfb(uchar Pidelayfb_)
 {
     Pidelayfb = Pidelayfb_;
     idelayfb = Pidelayfb / 128.0f;
 }
 
 
-void Reverb::sethpf(unsigned char Phpf_)
+void Reverb::sethpf(uchar Phpf_)
 {
     Phpf = Phpf_;
     if (Phpf == 0)
@@ -362,15 +356,17 @@ void Reverb::sethpf(unsigned char Phpf_)
         if (hpf)
             delete hpf;
         hpf = NULL;
-    } else {
+    }
+    else
+    {
         hpffr.setTargetValue(expf(powf(Phpf / 127.0f, 0.5f) * logf(10000.0f)) + 20.0f);
         if (hpf == NULL)
-            hpf = new AnalogFilter(3, hpffr.getValue(), 1, 0, synth);
+            hpf = new AnalogFilter(synth, TOPLEVEL::filter::High2, hpffr.getValue(), 1, 0);
     }
 }
 
 
-void Reverb::setlpf(unsigned char Plpf_)
+void Reverb::setlpf(uchar Plpf_)
 {
     Plpf = Plpf_;
     if (Plpf == 127)
@@ -378,51 +374,57 @@ void Reverb::setlpf(unsigned char Plpf_)
         if (lpf)
             delete lpf;
         lpf = NULL;
-    } else {
+    }
+    else
+    {
         lpffr.setTargetValue(expf(powf(Plpf / 127.0f, 0.5f) * logf(25000.0f)) + 40.0f);
         if (!lpf)
-            lpf = new AnalogFilter(2, lpffr.getValue(), 1, 0, synth);
+            lpf = new AnalogFilter(synth, TOPLEVEL::filter::Low2, lpffr.getValue(), 1, 0);
     }
 }
 
 
-void Reverb::settype(unsigned char Ptype_)
+void Reverb::settype(uchar Ptype_)
 {
     Ptype = Ptype_;
-    const int NUM_TYPES = 3;
     if (Ptype >= NUM_TYPES)
         Ptype = NUM_TYPES - 1;
 
+    cleanup(); // invokes setupPipelines()
+}
+
+
+void Reverb::setupPipelines()
+{
     int combtunings[NUM_TYPES][REV_COMBS] = {
         { 0, 0, 0, 0, 0, 0, 0, 0 }, // this is unused (for random)
 
         // Freeverb by Jezar at Dreampoint
         { 1116, 1188, 1277, 1356, 1422, 1491, 1557, 1617 },
-        // duplicate of Freeverb by Jezar at Dreampoint
         { 1116, 1188, 1277, 1356, 1422, 1491, 1557, 1617 }
     };
 
     int aptunings[NUM_TYPES][REV_APS] = {
         { 0, 0, 0, 0 },         // this is unused (for random)
         { 225, 341, 441, 556 }, // Freeverb by Jezar at Dreampoint
-        { 225, 341, 441, 556 }  // duplicate of Freeverb by Jezar at Dreampoint
+        { 225, 341, 441, 556 }
     };
 
-    float samplerate_adjust = synth->samplerate_f / 44100.0f;
+    float samplerate_adjust = synth.samplerate_f / 44100.0f;
 
     // adjust the combs according to samplerate and room size
     for (int i = 0; i < REV_COMBS * 2; ++i)
     {
         float tmp;
         if (Ptype == 0)
-            tmp = 800.0f + synth->numRandom() * 1400.0f;
+            tmp = 800.0f + synth.numRandom() * 1400.0f;
         else
             tmp = combtunings[Ptype][i % REV_COMBS];
         tmp *= roomsize;
         if (i > REV_COMBS)
             tmp += 23.0f;
         tmp *= samplerate_adjust; // adjust the combs according to the samplerate
-        comblen[i] = int(tmp);
+        comblen[i] = size_t(tmp);
         if (comblen[i] < 10)
             comblen[i] = 10;
         combk[i] = 0;
@@ -438,7 +440,7 @@ void Reverb::settype(unsigned char Ptype_)
         float tmp;
         if (Ptype == 0)
         {
-            tmp = 500.0f + synth->numRandom() * 500.0f;
+            tmp = 500.0f + synth.numRandom() * 500.0f;
         }
         else
             tmp = aptunings[Ptype][i % REV_APS];
@@ -446,7 +448,7 @@ void Reverb::settype(unsigned char Ptype_)
         if (i > REV_APS)
             tmp += 23.0f;
         tmp *= samplerate_adjust; // adjust the combs according to the samplerate
-        aplen[i] = int(tmp);
+        aplen[i] = size_t(tmp);
         if (aplen[i] < 10)
             aplen[i] = 10;
         apk[i] = 0;
@@ -460,7 +462,7 @@ void Reverb::settype(unsigned char Ptype_)
     bandwidth = NULL;
     if (Ptype == 2)
     { // bandwidth
-        bandwidth = new Unison(synth->buffersize / 4 + 1, 2.0f, synth);
+        bandwidth = new Unison(synth.buffersize / 4 + 1, 2.0f, &synth);
         bandwidth->setSize(50);
         bandwidth->setBaseFrequency(1.0f);
         //TODO the size of the unison buffer may be too small, though this has
@@ -468,12 +470,10 @@ void Reverb::settype(unsigned char Ptype_)
         //As this cannot be resized in a RT context, a good upper bound should
         //be found
     }
-    settime(Ptime);
-    cleanup();
 }
 
 
-void Reverb::setroomsize(unsigned char Proomsize_)
+void Reverb::setroomsize(uchar Proomsize_)
 {
     Proomsize = Proomsize_;
     if (!Proomsize)
@@ -487,7 +487,7 @@ void Reverb::setroomsize(unsigned char Proomsize_)
 }
 
 
-void Reverb::setbandwidth(unsigned char Pbandwidth_)
+void Reverb::setbandwidth(uchar Pbandwidth_)
 {
     Pbandwidth = Pbandwidth_;
     float v = Pbandwidth / 127.0f;
@@ -496,33 +496,33 @@ void Reverb::setbandwidth(unsigned char Pbandwidth_)
 }
 
 
-void Reverb::setpreset(unsigned char npreset)
+void Reverb::setpreset(uchar npreset)
 {
     if (npreset < 0xf)
     {
-        if (npreset >= NUM_PRESETS)
-            npreset = NUM_PRESETS - 1;
-        for (int n = 0; n < PRESET_SIZE; ++n)
-            changepar(n, presets[npreset][n]);
+        if (npreset >= reverbNUM_PRESETS)
+            npreset = reverbNUM_PRESETS - 1;
+        for (int n = 0; n < reverbPRESET_SIZE; ++n)
+            changepar(n, reverbPresets[npreset][n]);
         if (insertion)
-            changepar(0, presets[npreset][0] / 2); // lower the volume if this is insertion effect
+            changepar(0, reverbPresets[npreset][0] / 2); // lower the volume if this is insertion effect
         Ppreset = npreset;
     }
     else
     {
-        unsigned char preset = npreset & 0xf;
-        unsigned char param = npreset >> 4;
+        uchar preset = npreset & 0xf;
+        uchar param = npreset >> 4;
         if (param == 0xf)
             param = 0;
-        changepar(param, presets[preset][param]);
+        changepar(param, reverbPresets[preset][param]);
         if (insertion && (param == 0))
-            changepar(0, presets[preset][0] / 2);
+            changepar(0, reverbPresets[preset][0] / 2);
     }
     Pchanged = false;
 }
 
 
-void Reverb::changepar(int npar, unsigned char value)
+void Reverb::changepar(int npar, uchar value)
 {
     if (npar == -1)
     {
@@ -575,7 +575,7 @@ void Reverb::changepar(int npar, unsigned char value)
 }
 
 
-unsigned char Reverb::getpar(int npar)
+uchar Reverb::getpar(int npar) const
 {
     switch (npar)
     {
@@ -611,9 +611,9 @@ float Revlimit::getlimits(CommandBlock *getData)
     int min = 0;
     int max = 127;
 
-    int def = presets[presetNum][control];
-    unsigned char canLearn = TOPLEVEL::type::Learnable;
-    unsigned char isInteger = TOPLEVEL::type::Integer;
+    int def = reverbPresets[presetNum][control];
+    uchar canLearn = TOPLEVEL::type::Learnable;
+    uchar isInteger = TOPLEVEL::type::Integer;
 
     switch (control)
     {
