@@ -6,7 +6,7 @@
     Copyright 2009-2010, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -70,7 +70,8 @@ inline void WaveShapeSamples::waveShapeSmps(int n, float *smps, unsigned char ty
                     smps[i] = (smps[i] - powf(smps[i], 3.0f)) * 3.0f;
                     if (ws < 1.0f)
                         smps[i] /= ws;
-                } else
+                }
+                else
                     smps[i] = 0.0f;
             }
             break;
