@@ -1,10 +1,10 @@
 /*
     MiscGui.cpp - common link between GUI and synth
 
-    Copyright 2016-2021 Will Godfrey & others
+    Copyright 2016-2023 Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -22,15 +22,22 @@
 #include "Misc/SynthEngine.h"
 #include "Misc/TextMsgBuffer.h"
 #include "Misc/NumericFuncs.h"
+#include "Params/RandomWalk.h"
 #include "MiscGui.h"
 #include "MasterUI.h"
 
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
 
+#include <FL/platform.H>
 #include <cairo.h>
 #include <cairo-xlib.h>
 
+#include <iostream>
+
+using std::to_string;
+using std::ostringstream;
+
 using func::bpm2text;
 using func::power;
 
@@ -48,7 +55,6 @@ float collect_readData(SynthEngine *synth, float value, unsigned char control, u
     else if (request != UNUSED)
         action |= request;
     CommandBlock putData;
-
     putData.data.value = value;
     putData.data.type = type;
     putData.data.source = action;
@@ -60,23 +66,17 @@ float collect_readData(SynthEngine *synth, float value, unsigned char control, u
     putData.data.parameter = parameter;
     putData.data.offset = offset;
     putData.data.miscmsg = miscmsg;
-    float result;
-    if (miscmsg != NO_MSG)
-    {
-        synth->interchange.readAllData(&putData);
-        result = putData.data.miscmsg;
-    }
-    else
-        result = synth->interchange.readAllData(&putData);
+    float result = synth->interchange.readAllData(putData);
+    if (miscmsg != NO_MSG) // outgoing value - we want to read this text
+        result = putData.data.miscmsg; // returned message ID
     return result;
-
 }
 
-void collect_data(SynthEngine *synth, float value, unsigned char action, unsigned char type, unsigned char control, unsigned char part, unsigned char kititem, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char offset, unsigned char miscmsg)
+void collect_writeData(SynthEngine *synth, float value, unsigned char action, unsigned char type, unsigned char control, unsigned char part, unsigned char kititem, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char offset, unsigned char miscmsg)
 {
     if (part < NUM_MIDI_PARTS && engine == PART::engine::padSynth)
     {
-        if (collect_readData(synth, 0, PART::control::partBusy, part))
+        if (collect_readData(synth, 0, TOPLEVEL::control::partBusy, part))
         {
             alert(synth, "Part " + to_string(part + 1) + " is busy");
             return;
@@ -108,9 +108,7 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
                 // check range & if learnable
                 float newValue;
                 putData.data.type = 3 | TOPLEVEL::type::Limits;
-                newValue = synth->interchange.readAllData(&putData);
-                //if (newValue != value)
-                    //std::cout << "Gui limits " << value <<" to " << newValue << std::endl;
+                newValue = synth->interchange.readAllData(putData);
                 if (Fl::event_state(FL_CTRL) != 0)
                 {
                     if (putData.data.type & TOPLEVEL::type::Learnable)
@@ -125,7 +123,7 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
                         type = TOPLEVEL::type::Learnable;
                     }
                 }
-                else
+                else if (insert != TOPLEVEL::insert::filterGroup  || parameter == UNUSED)
                 {
                     putData.data.value = newValue;
                     type = TOPLEVEL::type::Write;
@@ -142,7 +140,7 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
 
     putData.data.type = type;
     putData.data.source = action;
-    //cout << "collect_data value " << value << "  action " << int(action)  << "  type " << int(type) << "  control " << int(control) << "  part " << int(part) << "  kit " << int(kititem) << "  engine " << int(engine) << "  insert " << int(insert)  << "  par " << int(parameter) << " offset " << int(offset) << " msg " << int(miscmsg) << endl;
+
     if (!synth->interchange.fromGUI.write(putData.bytes))
         synth->getRuntime().Log("Unable to write to fromGUI buffer.");
 }
@@ -167,43 +165,43 @@ string input_text(SynthEngine *synth, string label, string text)
     return synth->getGuiMaster()->setinput(label, text);
 }
 
-//static int keycheck = 0;
+
+GuiUpdates::GuiUpdates(InterChange& _interChange, InterfaceAnchor&& connectionData)
+    : interChange{_interChange}
+    , anchor{std::move(connectionData)}
+{ }
+
+
 void GuiUpdates::read_updates(SynthEngine *synth)
 {
     CommandBlock getData;
     while (synth->interchange.toGUI.read(getData.bytes))
     {
-        Fl::lock();
         decode_updates(synth, &getData);
-        Fl::unlock();
     }
-    // and pull up to 5 entries from log
-    for (int i = 0; !synth->getRuntime().LogList.empty() && i < 5; ++i)
-    {
-        synth->getGuiMaster()->Log(synth->getRuntime().LogList.front());
-        synth->getRuntime().LogList.pop_front();
-    }
-    // direct key actions (not for LV2)
-    /*if (!synth->getIsLV2Plugin() && Fl::event_ctrl() && Fl::event_alt() && keycheck == 0)
-    {
-        if (Fl::event_key('H')) // immediate shutdown
-            synth->getGuiMaster()->masterwindow->do_callback();
-        else if (Fl::event_key('R')) // resize to fit
-        {
-            if(!synth->getIsLV2Plugin())
-            for (unsigned int id = 0; id < 31; ++id)
-            {
-                SynthEngine *_synth = synth->getSynthFromId(id);
-                if (_synth != synth)
-                    _synth->getGuiMaster()->rescaleMain();
-            }
-            synth->getGuiMaster()->rescaleMain();
-        }
-        keycheck = 90; // we need this to stop a stream of repeats
 
+    // test refresh time
+    /*
+    static int count = 0;
+    static int toggle = false;
+    ++count;
+    if (count > 30)
+    {
+        count = 0;
+        toggle = !toggle;
+        if (toggle)
+            synth->getRuntime().Log("Tick");
+        else
+            synth->getRuntime().Log("tock");
+    }
+    */
+
+    // and pull up to 5 entries from log
+    for (int i = 0; !synth->getRuntime().logList.empty() && i < 5; ++i)
+    {
+        synth->getGuiMaster()->Log(synth->getRuntime().logList.front());
+        synth->getRuntime().logList.pop_front();
     }
-    else if (keycheck > 0)
-        --keycheck;*/
 }
 
 
@@ -248,15 +246,46 @@ void GuiUpdates::decode_envelope(SynthEngine *synth, CommandBlock *getData)
 
 void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
 {
-    unsigned char control = getData->data.control;
-    unsigned char npart = getData->data.part;
-    unsigned char kititem = getData->data.kit;
-    unsigned char engine = getData->data.engine;
-    unsigned char insert = getData->data.insert;
-    unsigned char parameter = getData->data.parameter;
-    unsigned char miscmsg = getData->data.miscmsg;
+    uchar control   = getData->data.control;
+    uchar npart     = getData->data.part;
+    uchar kititem   = getData->data.kit;
+    uchar engine    = getData->data.engine;
+    uchar insert    = getData->data.insert;
+    uchar parameter = getData->data.parameter;
+    uchar miscmsg   = getData->data.miscmsg;
 
-//        cout << "Con " << int(control) << "  Kit " << int(kititem) << "  Eng " << int(engine) << "  Ins " << int(insert) << endl;
+    if (control == TOPLEVEL::control::dataExchange)
+    {
+        if (npart == TOPLEVEL::section::message)
+        {// push data messages via GuiDataExchange -> deliver directly to MirrorData receivers
+            synth->interchange.guiDataExchange.dispatchUpdates(*getData);
+            return;
+        }
+        else if (npart == TOPLEVEL::section::main)
+        {// Global refresh when SynthEngine becomes ready
+            synth->getGuiMaster()->refreshInit();
+    }   }
+
+    if (control == TOPLEVEL::control::copyPaste)
+    {
+        if (getData->data.type == TOPLEVEL::type::Adjust)
+            return; // just looking
+        if (npart == TOPLEVEL::section::systemEffects || npart == TOPLEVEL::section::insertEffects)
+        {
+            synth->getGuiMaster()->paste(getData);
+            return;
+        }
+        else if (npart <= TOPLEVEL::section::part64)
+        {
+            synth->getGuiMaster()->partui->paste(getData);
+            return;
+        }
+        else
+        {
+            std::cout << "no copy/paste valid" << std::endl;
+            return;
+        }
+    }
 
     if (control == TOPLEVEL::control::textMessage) // just show a non-modal message
     {
@@ -294,11 +323,18 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         return;
     }
 
-    if (kititem >= EFFECT::type::none && kititem != UNUSED) // effects
+    bool allowPartUpdate = false;
+    int GUIpart = synth->getGuiMaster()->npartcounter->value() -1;
+    if (GUIpart == npart)
     {
+        allowPartUpdate = true;
+    }
+
+    if (npart != TOPLEVEL::section::main && kititem >= EFFECT::type::none && kititem < EFFECT::type::count) // effects
+    { // maybe we should go to main first?
         if (npart == TOPLEVEL::section::systemEffects)
-        {
-            if (engine != synth->getGuiMaster()->nsyseff)
+        {   // note: prior to processing the returns, a push-update has been sent to the effect-UI
+            if (engine != synth->getGuiMaster()->syseffectui->effNum())
                 return;
             if (insert == TOPLEVEL::insert::filterGroup) // dynefilter filter insert
                 synth->getGuiMaster()->syseffectui->fwin_filterui->returns_update(getData);
@@ -307,16 +343,16 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         }
         else if (npart == TOPLEVEL::section::insertEffects)
         {
-            if (engine != synth->getGuiMaster()->ninseff)
+            if (engine != synth->getGuiMaster()->inseffectui->effNum())
                 return;
             if (insert == TOPLEVEL::insert::filterGroup) // dynefilter filter insert
                 synth->getGuiMaster()->inseffectui->fwin_filterui->returns_update(getData);
             else
                 synth->getGuiMaster()->inseffectui->returns_update(getData);
         }
-        else if (npart < NUM_MIDI_PARTS)
+        else if (npart < NUM_MIDI_PARTS && allowPartUpdate)
         {
-            if (engine != synth->getGuiMaster()->partui->ninseff)
+            if (engine != synth->getGuiMaster()->partui->inseffectui->effNum())
                 return;
             if (insert == TOPLEVEL::insert::filterGroup) // dynefilter filter insert
                 synth->getGuiMaster()->partui->inseffectui->fwin_filterui->returns_update(getData);
@@ -331,18 +367,28 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         synth->getGuiMaster()->configui->returns_update(getData);
         return;
     }
+
     if (npart == TOPLEVEL::section::main && control == MAIN::control::exportPadSynthSamples) // special case
     {
         npart = parameter & 0x3f;
         getData->data.part = npart;
     }
+
     if (npart >= TOPLEVEL::section::main) // main / sys / ins
     {
         synth->getGuiMaster()->returns_update(getData);
         return;
     }
+    /*
+     * we are managing some part-related controls from here
+    */
+    if (npart < NUM_MIDI_PARTS && (kititem & engine & insert) == UNUSED && allowPartUpdate)
+    {
+        if (synth->getGuiMaster()->part_group_returns(getData))
+            return;
+    }
 
-    if (npart >= NUM_MIDI_PARTS)
+    if (npart >= NUM_MIDI_PARTS || !allowPartUpdate)
         return; // invalid part number
 
     if (kititem >= NUM_KIT_ITEMS && kititem != UNUSED)
@@ -395,8 +441,11 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->padnoteui->filterui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
+                case TOPLEVEL::insert::envelopePointChangeDt:
+                case TOPLEVEL::insert::envelopePointChangeVal:
                     switch(parameter)
                     {
                         case TOPLEVEL::insertType::amplitude:
@@ -416,7 +465,7 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
 
                 case TOPLEVEL::insert::oscillatorGroup:
                 case TOPLEVEL::insert::harmonicAmplitude:
-                case TOPLEVEL::insert::harmonicPhaseBandwidth:
+                case TOPLEVEL::insert::harmonicPhase:
                     if (synth->getGuiMaster()->partui->padnoteui->oscui)
                         synth->getGuiMaster()->partui->padnoteui->oscui->returns_update(getData);
                     break;
@@ -426,8 +475,6 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->padnoteui->resui->returns_update(getData);
                     break;
             }
-            if (getData->data.offset == 0)
-                synth->getGuiMaster()->partui->padnoteui->applycolour(FL_RED);
         }
         else if (miscmsg != NO_MSG)
         {
@@ -445,9 +492,29 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                     if (synth->getGuiMaster()->partui->subnoteui->filterui)
                         synth->getGuiMaster()->partui->subnoteui->filterui->returns_update(getData);
                     break;
+                case TOPLEVEL::insert::LFOgroup:
+                    switch(parameter)
+                    {
+                        case TOPLEVEL::insertType::amplitude:
+                            if (synth->getGuiMaster()->partui->subnoteui->amplfo)
+                                synth->getGuiMaster()->partui->subnoteui->amplfo->returns_update(getData);
+                            break;
+                        case TOPLEVEL::insertType::frequency:
+                            if (synth->getGuiMaster()->partui->subnoteui->freqlfogroup)
+                                synth->getGuiMaster()->partui->subnoteui->freqlfogroup->returns_update(getData);
+                            break;
+                        case TOPLEVEL::insertType::filter:
+                            if (synth->getGuiMaster()->partui->subnoteui->filterlfo)
+                                synth->getGuiMaster()->partui->subnoteui->filterlfo->returns_update(getData);
+                            break;
+                    }
+                    break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
+                case TOPLEVEL::insert::envelopePointChangeDt:
+                case TOPLEVEL::insert::envelopePointChangeVal:
                     switch(parameter)
                     {
                         case TOPLEVEL::insertType::amplitude:
@@ -470,7 +537,7 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                     break;
                 case UNUSED:
                 case TOPLEVEL::insert::harmonicAmplitude:
-                case TOPLEVEL::insert::harmonicPhaseBandwidth:
+                case TOPLEVEL::insert::harmonicBandwidth:
                     synth->getGuiMaster()->partui->subnoteui->returns_update(getData);
                     break;
             }
@@ -512,15 +579,18 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                     case TOPLEVEL::insert::envelopeGroup:
                         decode_envelope(synth, getData);
                         break;
-                    case TOPLEVEL::insert::envelopePoints:
+                    case TOPLEVEL::insert::envelopePointAdd:
+                    case TOPLEVEL::insert::envelopePointDelete:
                         decode_envelope(synth, getData);
                         break;
                     case TOPLEVEL::insert::envelopePointChange:
+                    case TOPLEVEL::insert::envelopePointChangeDt:
+                    case TOPLEVEL::insert::envelopePointChangeVal:
                         decode_envelope(synth, getData);
                         break;
                     case TOPLEVEL::insert::oscillatorGroup:
                     case TOPLEVEL::insert::harmonicAmplitude:
-                    case TOPLEVEL::insert::harmonicPhaseBandwidth:
+                    case TOPLEVEL::insert::harmonicPhase:
                         if (synth->getGuiMaster()->partui->adnoteui->advoice->oscedit)
                             synth->getGuiMaster()->partui->adnoteui->advoice->oscedit->returns_update(getData);
                         break;
@@ -560,8 +630,11 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->adnoteui->filterui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
+                case TOPLEVEL::insert::envelopePointChangeDt:
+                case TOPLEVEL::insert::envelopePointChangeVal:
                     switch(parameter)
                     {
                         case TOPLEVEL::insertType::amplitude:
@@ -589,8 +662,25 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     }
 }
 
-// static string freqBPMStr(float val)
-// This has been moved as bpm2text in Misc/NumericFuncs.h with other BPM shared functions
+
+// for setting slider peg colour
+int setSlider(float current, float normal)
+{
+    if (lrint(current) == lrint(normal))
+        return slider_peg_default;
+    else
+       return slider_peg_changed;
+}
+
+
+// for setting knob pointer colour
+int setKnob(float current, float normal)
+{
+    if ((current - normal) < 0.0005 && (normal - current) < 0.0005)
+        return knob_point;
+    else
+       return knob_point_change;
+}
 
 
 string convert_value(ValueType type, float val)
@@ -606,8 +696,8 @@ string convert_value(ValueType type, float val)
         case VC_pitchWheel:
             return(custom_value_units(-val,"",1));
 
-        case VC_percent127:
-            return(custom_value_units(val / 127.0f * 100.0f+0.05f,"%",1));
+        case VC_percent127: // removed offset to get zero W.G.
+            return(custom_value_units(val / 127.0f * 100.0f,"%",1));
 
         case VC_percent128:
             return(custom_value_units(val / 128.0f * 100.0f+0.05f,"%",1));
@@ -883,7 +973,7 @@ string convert_value(ValueType type, float val)
             f = -20.0f * logf(powf((1.0f / 127.0f), f)) / log(10.0f);
             s += variable_prec_units(f, "dB", 2);
             s += "\nVelocity/2 = ";
-            s += variable_prec_units(f/(-1 * log2(127)), "dB", 2);
+            s += variable_prec_units(f/(-1 * std::log2(127)), "dB", 2);
             return(s);
 
         case VC_BandWidth:
@@ -909,11 +999,43 @@ string convert_value(ValueType type, float val)
         case VC_SubBandwidthScale:
             if ((int)val == 0)
                 return "Constant";
-	    f = val / 64.0f * 3.0f;
+            f = val / 64.0f * 3.0f;
             return "Factor (100,10k): " +
                 variable_prec_units(power<10>(f), "", 4) + ", " +
                 variable_prec_units(powf(0.1,f), "x", 4);
 
+        case VC_XFadeUpdate:
+        {
+            unsigned int millisec = logDial2millisec(val);
+            if (millisec > 1000)
+                return variable_prec_units(float(millisec) / 1000, "sec", 1);
+            if (millisec > 0)
+                return variable_prec_units(float(millisec), "ms", 0);
+            else
+                return "off";
+            break;
+        }
+
+        case VC_Retrigger:
+        {
+            if (val > 0) val += 2300;
+            // in the UI we remove a socket of 200ms from the dial setting,
+            // to prevent the user from choosing overly fast retriggering
+            // 200ms correspond to the log10 setting of 2300
+            return convert_value(VC_XFadeUpdate, val);
+            break;
+        }
+
+        case VC_RandWalkSpread:
+        {
+            double spread = RandomWalk::param2spread(val);
+            if (spread > 1)
+                return variable_prec_units((spread - 1) * 100.0, "%", 1);
+            else
+                return "no random walk.";
+            break;
+        }
+
         case VC_FilterVelocitySense: // this is also shown graphically
             if ((int)val==127)
                 return("off");
@@ -1180,7 +1302,7 @@ void custom_graph_dimensions(ValueType vt, int& w, int& h)
 
 inline void grid(int x, int y, int w, int h, int sections)
 {
-        fl_color(FL_GRAY);
+        fl_color(tooltip_grid);
 
         int j = 1;
         int gDist = h / sections;
@@ -1214,7 +1336,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         /* Grid */
         grid(x0,y0,_w,_h, 4);
         /*Function curve*/
-        fl_color(FL_BLUE);
+        fl_color(tooltip_curve);
         if ((int)val == 127)
         {   // in this case velF will always return 1.0
             y = y0 - _h;
@@ -1237,7 +1359,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
     {
         p = power<10>((val - 32.0f) / 48.0f); //clearness param
         grid(x0,y0,_w,_h,10);
-        fl_color(FL_BLUE);
+        fl_color(tooltip_curve);
         fl_begin_line();
         x = 0;
         float frac = 1.0f / (float)_w;
@@ -1277,10 +1399,10 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         for (i = 0; i < 4; i++) /* 10x / 10%, 100x / 1% ... */
         {
             y = ry * (i + 1);
-            fl_color(169,169,169);
+            fl_color(tooltip_grid);
             fl_line(x0, cy - y, x0 + _w, cy - y);
             fl_line(x0, cy + y, x0 + _w, cy + y);
-            fl_color(0,0,0);
+            fl_color(tooltip_faint_text);
             fl_draw(xMarkers[i].c_str(), x0 - 28, (cy - y - 4), 24, 12,
                     Fl_Align(FL_ALIGN_RIGHT));
             fl_draw(xMarkers[i + 4].c_str(), x0 - 28, (cy + y - 4), 24, 12,
@@ -1289,7 +1411,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
 
         /* Hz lines */
 
-        fl_color(196,196,196); /* Lighter inner lines*/
+        fl_color(tooltip_grid); /* Lighter inner lines*/
 
         for (i = 10;i != 0; i *= 10)
         {
@@ -1309,9 +1431,9 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         for (i = 0; i < 4; i++) /* 20, 100, 1k, 10k */
         {
             x = x0 + (i == 0 ?  0 : ((float)i + 1 - lg1020) * rx);
-            fl_color(127,127,127); /* Darker boundary lines */
+            fl_color(tooltip_major_grid); /* Darker boundary lines */
             fl_line(x, y0, x, y0 - _h);
-            fl_color(FL_BLACK);
+            fl_color(tooltip_text);
             fl_draw(hzMarkers[i].c_str(), x - 20, y0 + 4, 40, 12,
                     Fl_Align(FL_ALIGN_CENTER));
         }
@@ -1319,11 +1441,11 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         fl_draw("Hz", x0 + _w, y0 + 4, 20, 12, Fl_Align(FL_ALIGN_LEFT));
 
         /* Vertical center line */
-        fl_color(64,64,64);
+        fl_color(38);
         fl_line(x0 - margin, cy, x0 + _w, cy);
 
         /* Function curve */
-        fl_color(FL_BLUE);
+        fl_color(tooltip_curve);
         if ((int)val == 0)
         {
             fl_line(x0, cy, x0 + _w, cy);
@@ -1417,3 +1539,16 @@ ValueType getFilterFreqTrackType(int offset)
     }
 }
 
+/** convert a milliseconds value to a logarithmic dial setting */
+int millisec2logDial(unsigned int ms)
+{
+    return ms==0? -1
+                : log10f(float(ms)) * 1000;
+}
+
+/** convert setting from a logarithmic dial back to milliseconds */
+unsigned int logDial2millisec(int dial)
+{
+    return dial<0? 0
+                 : power<10>(dial / 1000.0f) + 0.5;
+}
