@@ -6,10 +6,10 @@
     Copyright 2009-2010, Alan Calvert
     Copyright 2016 Will Godfrey
     Copyright 2017 Jesper Lloyd
-    Coyright 2020 - 2021, Will Godfrey & others
+    Copyright 2020-2025, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -32,12 +32,24 @@
 #include <FL/fl_draw.H>
 #include <FL/Fl_Tooltip.H>
 #include <FL/Fl_Group.H>
-#include <FL/x.H>
+
+#include <FL/platform.H>
 #include <cairo.h>
 #include <cairo-xlib.h>
 
 using func::limit;
 
+// Prior to fltk 1.4, there was no built in support for handling screen scales.
+// This may not be the correct way to handle the scale factor.
+inline float get_scale_factor(Fl_Widget *widget) {
+#if FL_API_VERSION > 10400
+    Fl_Window* window = widget->window();
+    return Fl::screen_scale(window ? window->screen_num() : 0);
+#else
+    return 1.0f;
+#endif
+}
+
 WidgetPDial::WidgetPDial(int x,int y, int w, int h, const char *label) : Fl_Dial(x,y,w,h,label)
 {
     Fl_Group *save = Fl_Group::current();
@@ -89,7 +101,6 @@ int WidgetPDial::handle(int event)
 
     double dragsize, v, min = minimum(), max = maximum();
     int my, mx;
-
     int res = 0;
 
     switch (event)
@@ -130,7 +141,7 @@ int WidgetPDial::handle(int event)
         my = - Fl::event_dy();
         dragsize = 25.0f;
         if (Fl::event_state(FL_CTRL) != 0)
-            dragsize *= 10;
+            dragsize *= 5; // halved this for better fine resolution
         value(limit(value() + my / dragsize * (max - min), min, max));
         value_damage();
         if (this->when() != 0)
@@ -170,16 +181,20 @@ void WidgetPDial::drawgradient(int cx,int cy,int sx,double m1,double m2)
 
 void WidgetPDial::draw()
 {
-    int cx = x(), cy = y(), sx = w(), sy = h();
+    float scale = get_scale_factor(this);
+    int cx = x() * scale, cy = y() * scale, sx = w() * scale, sy = h() * scale;
     double d = (sx>sy)?sy:sx; // d = the smallest side -2
     double dh = d/2;
-    fl_color(170,170,170);
-    fl_pie(cx - 2, cy - 2, d + 4, d + 4, 0, 360);
+       /*
+        * doing away with the fixed outer band. It's out of place now!
+        * fl_color(170,170,170);
+        * fl_pie(cx - 2, cy - 2, d + 4, d + 4, 0, 360);
+        */
     double val = (value() - minimum()) / (maximum() - minimum());
     cairo_t *cr;
     cairo_surface_t* Xsurface = cairo_xlib_surface_create
-        (fl_display, fl_window, fl_visual->visual,Fl_Window::current()->w(),
-         Fl_Window::current()->h());
+        (fl_display, fl_window, fl_visual->visual,Fl_Window::current()->w() * scale,
+         Fl_Window::current()->h() * scale);
     cr = cairo_create (Xsurface);
     cairo_translate(cr,cx+dh,cy+dh);
     //relative lengths of the various parts:
@@ -187,27 +202,50 @@ void WidgetPDial::draw()
     double rCout = 13.0/35;
     double rHand = 8.0/35;
     double rGear = 15.0/35;
+
+    unsigned char r,g,b;
     //drawing base dark circle
+    Fl::get_color(knob_ring, r, g, b); // 51,51,51
     if (active_r())
     {
-        cairo_pattern_create_rgb(51.0/255,51.0/255,51.0/255);
-    } else {
+       /*
+        * cairo_pattern_create_rgb(r/255.0,g/255.0,b/255.0);
+        * The above line seems to be wrong and draws black
+        * regardless of the selection.
+        * The line below works.
+        * Will G.
+        */
+        cairo_set_source_rgb(cr,r/255.0,g/255.0,b/255.0);
+    }
+    else
+    {
         cairo_set_source_rgb(cr,0.4,0.4,0.4);
     }
     cairo_arc(cr,0,0,dh,0,2*PI);
     cairo_fill(cr);
+
     cairo_pattern_t* pat;
+    Fl::get_color(knob_low, r, g, b);
+    float R1 = r/255.0; // 186
+    float G1 = g/255.0; // 198
+    float B1 = b/255.0; // 211
+
+    Fl::get_color(knob_high, r, g, b);
+    float R2 = r/255.0; // 231
+    float G2 = g/255.0; // 235
+    float B2 = b/255.0; // 239
+
     //drawing the inner circle:
     pat = cairo_pattern_create_linear(0.5*dh,0.5*dh,0,-0.5*dh);
-    cairo_pattern_add_color_stop_rgba (pat, 0, 0.8*186.0/255, 0.8*198.0/255, 0.8*211.0/255, 1);
-    cairo_pattern_add_color_stop_rgba (pat, 1, 231.0/255, 235.0/255, 239.0/255, 1);
+    cairo_pattern_add_color_stop_rgba (pat, 0, 0.8*R1, 0.8*G1, 0.8*B1, 1);
+    cairo_pattern_add_color_stop_rgba (pat, 1, R2, G2, B2, 1);
     cairo_set_source (cr, pat);
     cairo_arc(cr,0,0,d*rCout,0,2*PI);
     cairo_fill(cr);
     //drawing the outer circle:
     pat = cairo_pattern_create_radial(2.0/35*d,6.0/35*d,2.0/35*d,0,0,d*rCint);
-    cairo_pattern_add_color_stop_rgba (pat, 0, 231.0/255, 235.0/255, 239.0/255, 1);
-    cairo_pattern_add_color_stop_rgba (pat, 1, 186.0/255, 198.0/255, 211.0/255, 1);
+    cairo_pattern_add_color_stop_rgba (pat, 0, R2, G2, B2, 1);
+    cairo_pattern_add_color_stop_rgba (pat, 1, R1, G1, B1, 1);
     cairo_set_source (cr, pat);
     cairo_arc(cr,0,0,d*rCint,0,2*PI);
     cairo_fill(cr);
@@ -217,8 +255,11 @@ void WidgetPDial::draw()
         linewidth = 2;
     if (active_r())
     {
-        cairo_set_source_rgb(cr,0,197.0/255,245.0/255); //light blue
-    } else {
+        Fl::get_color(knob_lit, r, g, b); // 0, 197, 255
+        cairo_set_source_rgb(cr,r/255.0,g/255.0, b/255.0); //light blue
+    }
+    else
+    {
         cairo_set_source_rgb(cr,0.6,0.7,0.8);
     }
     cairo_set_line_width (cr, linewidth);
@@ -228,8 +269,13 @@ void WidgetPDial::draw()
     //drawing the hand:
     if (active_r())
     {
-        cairo_set_source_rgb(cr,61.0/255,61.0/255,61.0/255);
-    } else {
+        if (selection_color() == 8)
+            selection_color(knob_point);
+        Fl::get_color(selection_color(), r, g, b); // 61, 61, 61
+        cairo_set_source_rgb(cr,r/255.0,g/255.0,b/255.0);
+    }
+    else
+    {
         cairo_set_source_rgb(cr,111.0/255,111.0/255,111.0/255);
     }
     cairo_rotate(cr,val*3/2*PI+0.25*PI);
